@@ -1,23 +1,30 @@
 /*
- *  milkyplay/LoaderMOD.cpp
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -174,6 +181,10 @@ mp_sint32 LoaderMOD::load(XMFileBase& f, XModule* module)
 			return -8;
 	}
 	
+#ifdef VERBOSE
+	printf("Loading...\n");
+#endif
+	
 	mp_sint32 i, s = 0;
 	for (i = 0; i < header->insnum; i++) {
 		mp_ubyte insname[22];
@@ -190,6 +201,10 @@ mp_sint32 LoaderMOD::load(XMFileBase& f, XModule* module)
 		loopstart = f.readWord();
 		looplen = f.readWord();
 		
+#ifdef VERBOSE
+		printf("Ins %i, smplen: %i, loopstart: %i, looplen: %i\n", i, mot2int(smplen), mot2int(loopstart), mot2int(looplen));
+#endif
+		
 		memcpy(instr[i].name, insname, 22);
 
 		// valid sample?
@@ -418,6 +433,10 @@ mp_sint32 LoaderMOD::load(XMFileBase& f, XModule* module)
 	}
 
 	module->postProcessSamples();
+	
+#ifdef VERBOSE
+	printf("%i / %i\n", f.pos(), f.size());
+#endif
 
 	return 0;
 }
