@@ -1,23 +1,30 @@
 /*
- *  milkyplay/LoaderPSM.cpp
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -25,6 +32,7 @@
  *  MilkyPlay Module Loader: Version 1/2 of Epic MegaGames MASI
  */
 #include "Loaders.h"
+#include <stdio.h>
 
 static bool PATTTest(mp_ubyte* p, mp_sint32& size)
 {
@@ -243,11 +251,21 @@ mp_sint32 LoaderPSMv2::load(XMFileBase& f, XModule* module)
 						// helper pattern to set speed and tempo for that particular song
 						else
 						{
+							char patSig[10];
+							sprintf(patSig, "P%i  ", patIndex % 999);
+						
 							patterns[patIndex] = new mp_ubyte[20];
 							memset(patterns[patIndex],0,20);
+							memcpy(patterns[patIndex]+4, patSig, 4);
 							patternSizes[patIndex] = 20;
 							
 							mp_ubyte* pattern = patterns[patIndex];
+
+							pattern[0] = 0xFF;
+							pattern[1] = 0xFF;
+							pattern[2] = 0xFF;
+							pattern[3] = 0xFF;
+
 							pattern[8] = 0x01;	// just one row
 							pattern[10] = 0x0A;	// 10 bytes in size
 							pattern[12] = 0x10; // read effect + operand 
@@ -575,6 +593,7 @@ mp_sint32 LoaderPSMv2::load(XMFileBase& f, XModule* module)
 		
 		mp_ubyte* dstSlot = phead[i].patternData;
 		for (row = 0; row < phead[i].rows; row++)
+		{
 			for (mp_uint32 c = 0; c < maxChannels; c++)
 			{
 				const mp_ubyte* srcSlot = pattern+row*32*5+c*5;
@@ -802,6 +821,21 @@ mp_sint32 LoaderPSMv2::load(XMFileBase& f, XModule* module)
 				
 				dstSlot+=8;
 			}
+			
+		}
+		
+		bool dummyPattern = (*(patterns[i]) == 0xFF 
+							 && *(patterns[i]+1) == 0xFF  
+							 && *(patterns[i]+2) == 0xFF  
+							 && *(patterns[i]+3) == 0xFF);
+		
+		// mark song start with effect 0x1E + operand 0xFF
+		if (dummyPattern && phead[i].rows == 1)
+		{
+			phead[i].patternData[2] = XModule::SubSongMarkEffect;
+			phead[i].patternData[3] = XModule::SubSongMarkOperand;
+		}
+			
 	}
 		
 	delete[] pattern;
@@ -996,7 +1030,7 @@ mp_sint32 LoaderPSMv1::load(XMFileBase& f, XModule* module)
 			delete[] pattern;
 			return -7;				
 		}
-		memset(packed,size-4+5,0);
+		memset(packed,0,size-4+5);
 		
 		MEMREAD(packed,1,size-4);
 		
