@@ -1,23 +1,30 @@
 /*
- *  milkyplay/LoaderXM.cpp
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -454,7 +461,7 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 		mp_sint32 s = 0;
 		mp_sint32 e = 0;
 		for (y=0;y<header->insnum;y++) {
-		
+
 			// fixes MOOH.XM loading problems
 			// seems to store more instruments in the header than in the actual file
 			if (f.posWithBaseOffset() >= fileSize)
@@ -649,6 +656,7 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 					{
 						bool adpcm = (smp[s].res == 0xAD);
 					
+						mp_uint32 oldSize = smp[s].samplen;
 						if (smp[s].type&16) 
 						{
 							smp[s].samplen>>=1;
@@ -657,8 +665,9 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 						}
 						
 						if (module->loadModuleSample(f, s, 
-													  adpcm ? XModule::ST_PACKING_ADPCM : XModule::ST_DELTA, 
-													  adpcm ? (XModule::ST_PACKING_ADPCM | XModule::ST_16BIT) : (XModule::ST_DELTA | XModule::ST_16BIT)) != 0)
+													 adpcm ? XModule::ST_PACKING_ADPCM : XModule::ST_DELTA, 
+													 adpcm ? (XModule::ST_PACKING_ADPCM | XModule::ST_16BIT) : (XModule::ST_DELTA | XModule::ST_16BIT), 
+													 oldSize) != 0)
 							return -7;					
 						
 						if (adpcm)
@@ -700,6 +709,7 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 
 				if (smp[s].samplen)
 				{
+					mp_uint32 oldSize = smp[s].samplen;
 					if (smp[s].type&16) 
 					{
 						smp[s].samplen>>=1;
@@ -707,7 +717,7 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 						smp[s].looplen>>=1;
 					}
 					
-					if (module->loadModuleSample(f, s, XModule::ST_DEFAULT, XModule::ST_DELTA | XModule::ST_16BIT) != 0)
+					if (module->loadModuleSample(f, s, XModule::ST_DELTA, XModule::ST_DELTA | XModule::ST_16BIT, oldSize) != 0)
 						return -7;					
 				}
 				
@@ -767,6 +777,12 @@ mp_sint32 LoaderXM::load(XMFileBase& f, XModule* module)
 				}
 			}
 		}
+		
+		// correct loop type 0x03 (undefined)
+		// will become ping pong loop
+		// note that FT2 will refuse to load XM files with such a loop type
+		if ((smp[s].type & 0x3) == 0x3)
+			smp[s].type&=~1;		
 	}
 
 	// correct number of patterns if necessary, otherwise the post processing will remove
