@@ -1,23 +1,30 @@
 /*
- *  milkyplay/PlayerSTD.cpp
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -205,8 +212,8 @@ mp_sint32	PlayerSTD::getlogperiod(mp_sint32 note,mp_sint32 relnote,mp_sint32 fin
 
 PlayerSTD::PlayerSTD(mp_uint32 frequency,
 					 StatusEventListener* statusEventListener/* = NULL*/) : 
-	statusEventListener(statusEventListener),
 	PlayerBase(frequency),
+	statusEventListener(statusEventListener),
 	chninfo(NULL),
 	lastNumAllocatedChannels(-1)
 {
@@ -278,6 +285,9 @@ void PlayerSTD::timerHandler(mp_sint32 currentBeatPacket)
 	if (paused)
 		return;
 
+	if (statusEventListener)
+		statusEventListener->timerTickStarted(*this, *module);
+
 	mp_int64 dummy = (mp_int64)BPMCounter;
 	dummy+=(mp_int64)adder;
 	BPMCounter=(mp_sint32)dummy;
@@ -290,7 +300,14 @@ void PlayerSTD::timerHandler(mp_sint32 currentBeatPacket)
 #else
 		setActiveChannels(module->header.channum);
 #endif
+
+		if (statusEventListener)
+			statusEventListener->playerTickStarted(*this, *module);
+
 		tickhandler();
+
+		if (statusEventListener)
+			statusEventListener->playerTickEnded(*this, *module);
 	}
 	
 	if (module->header.flags & XModule::MODULE_AMSENVELOPES)
@@ -300,22 +317,6 @@ void PlayerSTD::timerHandler(mp_sint32 currentBeatPacket)
 	TModuleChannel* chnInf = chninfo;
 	for (mp_sint32 i = 0; i < initialNumChannels; i++)
 	{
-		// handle notes that are played from external source
-		// i.e. keyboard playback
-		if (chnInf->notePlayReadIndex < chnInf->notePlayWriteIndex)
-		{
-			mp_sint32 idx = chnInf->notePlayReadIndex & (TModuleChannel::NPQSIZE-1);
-			mp_sint32 note = chnInf->notePlayEntries[idx].note;
-			if (note)
-			{
-				playNoteInternal(i, note, 
-								chnInf->notePlayEntries[idx].ins, 
-								chnInf->notePlayEntries[idx].vol);
-				chnInf->notePlayEntries[idx].note = 0;
-			}
-			chnInf->notePlayReadIndex++;
-		}
-		
 		// track envelope position state
 		TPrEnv* env = &chnInf->venv;		
 		if ((env->envstruc && (env->envstruc->type & 1) && env->envstruc->num) &&
@@ -346,6 +347,9 @@ void PlayerSTD::timerHandler(mp_sint32 currentBeatPacket)
 		chnInf++;
 	}
 #endif
+
+	if (statusEventListener)
+		statusEventListener->timerTickStarted(*this, *module);
 }
 
 void PlayerSTD::restart(mp_uint32 startPosition/* = 0*/, mp_uint32 startRow/* = 0*/, bool resetMixer/* = true*/, const mp_ubyte* customPanningTable/* = NULL*/, bool playOneRowOnly/* = false*/)
@@ -488,11 +492,12 @@ void PlayerSTD::clearEffectMemory()
 void PlayerSTD::prenvelope(mp_sint32 c,TPrEnv *env,mp_sint32 keyon)
 {
 	if (env->envstruc!=NULL && (env->envstruc->type&1)) {
-		
-		if ((env->envstruc->type&2) && (env->a==env->envstruc->sustain) && (keyon)) 
+		// if we're sitting on a sustain point and key is on, we don't advance further
+		if ((env->envstruc->type&2) && (env->a==env->envstruc->sustain) && 
+			(env->step == env->envstruc->env[env->a][0]) && keyon) 
 			return;
 		
-		if ((env->step!=env->envstruc->env[env->b][0]) && (env->b < env->envstruc->num)) 
+		if ((env->step != env->envstruc->env[env->b][0]) && (env->b < env->envstruc->num)) 
 			env->step++;
 
 		if (env->step == env->envstruc->env[env->b][0]) {
@@ -2511,7 +2516,7 @@ void PlayerSTD::progressRow()
 					chnInf->vol = chnInf->tremorVol = chnInf->tremoloVol = module->smp[chnInf->smp].vol;
 					chnInf->hasTremolo = false; chnInf->finalTremoloVol = 0;
 				}
-				if ((playMode == PlayMode_FastTracker2) &&
+				if (playModeFT2 &&
 					(module->smp[chnInf->smp].flags&2)) 
 					chnInf->pan = module->smp[chnInf->smp].pan;	
 				if ((module->smp[chnInf->smp].flags&4)) 
@@ -3068,7 +3073,7 @@ nextrow:
 
 void PlayerSTD::halt()
 {
-	stopped = true;
+	halted = true;
 	BPMCounter = adder = 0;
 	if (resetOnStopFlag)
 		resetChannelsWithoutMuting();
@@ -3089,7 +3094,7 @@ bool PlayerSTD::grabChannelInfo(mp_sint32 chn, TPlayerChannelInfo& channelInfo)
 //////////////////////////////////////////////////////////////////////////////////////////
 // for MilkyTracker use
 //////////////////////////////////////////////////////////////////////////////////////////
-void PlayerSTD::playNoteInternal(mp_ubyte chn, mp_sint32 note, mp_sint32 i, mp_sint32 vol)
+void PlayerSTD::playNote(mp_ubyte chn, mp_sint32 note, mp_sint32 i, mp_sint32 vol/* = -1*/)
 {
 	if (!i)
 		return;
@@ -3136,6 +3141,12 @@ void PlayerSTD::playNoteInternal(mp_ubyte chn, mp_sint32 note, mp_sint32 i, mp_s
 					chnInf->ins = 0;
 					return;
 				}
+				
+				mp_sint32 finalNote = note + (mp_sint32)module->smp[chnInf->smp].relnote;
+					
+				// Within current note range?
+				if (!(finalNote >= 1 && finalNote <= 119))
+					return;
 			
 				chnInf->finetune=module->smp[chnInf->smp].finetune;
 				chnInf->relnote=module->smp[chnInf->smp].relnote;
@@ -3210,18 +3221,3 @@ void PlayerSTD::playNoteInternal(mp_ubyte chn, mp_sint32 note, mp_sint32 i, mp_s
 	}
 			
 }
-
-void PlayerSTD::playNote(mp_ubyte chn, 
-						 mp_sint32 note, mp_sint32 ins, mp_sint32 vol/* = -1*/)
-{	
-#ifdef MILKYTRACKER
-	TModuleChannel* chnInf = &chninfo[chn];
-	// fill ring buffer with note entries
-	// the callback will query these notes and play them 
-	mp_sint32 idx = chnInf->notePlayWriteIndex & (TModuleChannel::NPQSIZE-1);
-	chnInf->notePlayEntries[idx].ins = ins;
-	chnInf->notePlayEntries[idx].vol = vol;
-	chnInf->notePlayEntries[idx].note = note;
-	chnInf->notePlayWriteIndex++;
-#endif
-}
