@@ -1,23 +1,30 @@
 /*
- *  milkyplay/XModule.cpp
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -1555,32 +1562,38 @@ bool XModule::loadSample(XMFileBase& f,void* buffer,mp_uint32 size,mp_uint32 len
 	return true;
 }
 
-mp_sint32 XModule::loadModuleSample(XMFileBase& f, mp_sint32 index, mp_sint32 flags8/* = ST_DEFAULT*/, mp_sint32 flags16/* = ST_16BIT*/)
+mp_sint32 XModule::loadModuleSample(XMFileBase& f, mp_sint32 index, 
+									mp_sint32 flags8/* = ST_DEFAULT*/, mp_sint32 flags16/* = ST_16BIT*/,
+									mp_uint32 alternateSize/* = 0*/)
 {
 	if (smp[index].type & 16)
 	{
-		smp[index].sample = (mp_sbyte*)allocSampleMem(smp[index].samplen*2);
+		mp_uint32 finalSize = alternateSize ? alternateSize : smp[index].samplen*2;
+		
+		smp[index].sample = (mp_sbyte*)allocSampleMem(finalSize);
 		
 		if (smp[index].sample == NULL)
 		{
 			return -7;
 		}
 		
-		if (!loadSample(f,smp[index].sample, smp[index].samplen<<1, smp[index].samplen, flags16))
+		if (!loadSample(f,smp[index].sample, finalSize, smp[index].samplen, flags16))
 		{
 			return -7;
 		}
 	}
 	else
 	{
-		smp[index].sample = (mp_sbyte*)allocSampleMem(smp[index].samplen);
+		mp_uint32 finalSize = alternateSize ? alternateSize : smp[index].samplen;
+		
+		smp[index].sample = (mp_sbyte*)allocSampleMem(finalSize);
 		
 		if (smp[index].sample == NULL)
 		{
 			return -7;
 		}
 		
-		if (!loadSample(f,smp[index].sample, smp[index].samplen, smp[index].samplen, flags8))
+		if (!loadSample(f,smp[index].sample, finalSize, smp[index].samplen, flags8))
 		{
 			return -7;
 		}		
@@ -1589,7 +1602,6 @@ mp_sint32 XModule::loadModuleSample(XMFileBase& f, mp_sint32 index, mp_sint32 fl
 	return 0;
 }
 
-
 mp_sint32 XModule::loadModuleSamples(XMFileBase& f, mp_sint32 flags8/* = ST_DEFAULT*/, mp_sint32 flags16/* = ST_16BIT*/)
 {
 	for (mp_sint32 i = 0; i < header.smpnum; i++) 
@@ -1947,7 +1959,7 @@ const char* XModule::identifyModule(const mp_ubyte* buffer)
 mp_sint32 XModule::loadModule(const SYSCHAR* fileName, bool scanForSubSongs/* = false*/)
 {
 	XMFile f(fileName);
-	return loadModule(f, scanForSubSongs); 
+	return f.isOpen() ? loadModule(f, scanForSubSongs) : -8; 
 }
 
 mp_sint32 XModule::loadModule(XMFileBase& f, bool scanForSubSongs/* = false*/)
@@ -2344,6 +2356,17 @@ void XModule::buildSubSongTable()
 									break;
 								}
 								
+								case SubSongMarkEffect:
+								{
+									if (eop == SubSongMarkOperand)
+									{
+										subSongPositions[numSubSongs*2+1] = poscntMax;
+										numSubSongs++;
+										breakMain = true;
+										continue;
+									}
+								}
+								
 								case 0x2B:
 								{
 									pjump = 1;
