@@ -1,23 +1,30 @@
 /*
- *  milkyplay/XModule.h
+ * Copyright (c) 2009, The MilkyTracker Team.
+ * All rights reserved.
  *
- *  Copyright 2008 Peter Barth
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This file is part of Milkytracker.
- *
- *  Milkytracker is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Milkytracker is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Milkytracker.  If not, see <http://www.gnu.org/licenses/>.
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+ * - Neither the name of the <ORGANIZATION> nor the names of its contributors
+ *   may be used to endorse or promote products derived from this software
+ *   without specific prior written permission.
  *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -290,7 +297,7 @@ class XModule
 public:
 	enum
 	{
-		IdentificationBufferSize = 2048
+		IdentificationBufferSize	= 2048,
 	};
 
 	///////////////////////////////////////////////////////
@@ -411,14 +418,19 @@ public:
 	///////////////////////////////////////////////////////
 	// load sample into memory							 //
 	///////////////////////////////////////////////////////
-	static bool			loadSample(XMFileBase& f, void* buffer, mp_uint32 size, mp_uint32 length,mp_sint32 flags = ST_DEFAULT);
+	static bool			loadSample(XMFileBase& f, void* buffer, 
+								   mp_uint32 size, mp_uint32 length, 
+								   mp_sint32 flags = ST_DEFAULT);
 	
 	///////////////////////////////////////////////////////
 	// load a bunch of samples into memory				 //
 	///////////////////////////////////////////////////////
-	mp_sint32			loadModuleSample(XMFileBase& f, mp_sint32 index, mp_sint32 flags8 = ST_DEFAULT, mp_sint32 flags16 = ST_16BIT);
-
-	mp_sint32			loadModuleSamples(XMFileBase& f, mp_sint32 flags8 = ST_DEFAULT, mp_sint32 flags16 = ST_16BIT);
+	mp_sint32			loadModuleSample(XMFileBase& f, mp_sint32 index,
+										 mp_sint32 flags8 = ST_DEFAULT, mp_sint32 flags16 = ST_16BIT,
+										 mp_uint32 alternateSize = 0);
+	
+	mp_sint32			loadModuleSamples(XMFileBase& f, 
+										  mp_sint32 flags8 = ST_DEFAULT, mp_sint32 flags16 = ST_16BIT);
 	
 	static void			convertXMVolumeEffects(mp_ubyte volume, mp_ubyte& eff, mp_ubyte& op);
 	
@@ -532,7 +544,10 @@ public:
 		NOTE_LAST	= 120,
 		NOTE_OFF	= 121,
 		NOTE_CUT	= 122,
-		NOTE_FADE	= 123
+		NOTE_FADE	= 123,
+
+		SubSongMarkEffect	= 0x1E,
+		SubSongMarkOperand	= 0xFF,
 	};
 		
 	TXMHeader		header;		// module header
