@@ -1,7 +1,7 @@
 /*
  *  ppui/ListBox.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -56,15 +56,12 @@ void PPListBox::initialize()
 		hScrollbar->show(!autoHideHScroll);
 }
 
-PPListBox::PPListBox(pp_int32 id, 
-				 PPScreen* parentScreen, 
-				 EventListenerInterface* eventListener, 
-				 PPPoint location, 
-				 PPSize size, 
-				 bool border/*= true*/, 
-				 bool editable/*= false*/, 
-				 bool scrollable/* = true*/,
-				 bool showSelectionAlways/*= false*/) :
+PPListBox::PPListBox(pp_int32 id, PPScreen* parentScreen, EventListenerInterface* eventListener, 
+					 const PPPoint& location, const PPSize& size, 
+					 bool border/*= true*/, 
+					 bool editable/*= false*/, 
+					 bool scrollable/* = true*/,
+					 bool showSelectionAlways/*= false*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
 	borderColor(&PPUIConfig::getInstance()->getColor(PPUIConfig::ColorListBoxBorder)),
 	backGroundButtonColor(&PPUIConfig::getInstance()->getColor(PPUIConfig::ColorListBoxBackground)),
@@ -363,7 +360,7 @@ PPListBox::SelectReturnCodes PPListBox::select(const PPPoint* p)
 	return SelectReturnCodeDefault;
 }
 
-pp_int32 PPListBox::callEventListener(PPEvent* event)
+pp_int32 PPListBox::dispatchEvent(PPEvent* event)
 { 
 	if (eventListener == NULL)
 		return -1;
@@ -450,7 +447,7 @@ pp_int32 PPListBox::callEventListener(PPEvent* event)
 				if (controlCaughtByLMouseButton)
 					break;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				controlCaughtByRMouseButton = true;
 			}
 			// Clicked on vertical scrollbar -> route event to scrollbar and catch scrollbar control
@@ -459,7 +456,7 @@ pp_int32 PPListBox::callEventListener(PPEvent* event)
 				if (controlCaughtByLMouseButton)
 					break;
 				caughtControl = vScrollbar;
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByRMouseButton = true;
 			}
 			else if (rightButtonConfirm)
@@ -524,7 +521,7 @@ dragorautoscroll:
 				}
 			}
 			else
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 
 			break;
 		}
@@ -541,7 +538,7 @@ dragorautoscroll:
 				if (controlCaughtByRMouseButton)
 					break;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				controlCaughtByLMouseButton = true;
 			}
 			// Clicked on vertical scrollbar -> route event to scrollbar and catch scrollbar control
@@ -550,7 +547,7 @@ dragorautoscroll:
 				if (controlCaughtByRMouseButton)
 					break;
 				caughtControl = vScrollbar;
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByLMouseButton = true;
 			}
 			// Clicked on text -> select text
@@ -572,7 +569,7 @@ dragorautoscroll:
 			controlCaughtByRMouseButton = false;
 			if (caughtControl && !controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;			
 				break;
 			}
@@ -688,7 +685,7 @@ placeCursor:
 
 			if (!controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;			
 				break;
 			}
@@ -960,7 +957,7 @@ skiprefresh:
 			if (caughtControl == NULL)
 				break;
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			break;
 
 	}
@@ -1108,7 +1105,7 @@ pp_int32 PPListBox::handleEvent(PPObject* sender, PPEvent* event)
 	return 0;
 }
 
-void PPListBox::setSize(PPSize size)
+void PPListBox::setSize(const PPSize& size)
 {
 	PPControl::setSize(size);
 	
@@ -1127,7 +1124,7 @@ void PPListBox::setSize(PPSize size)
 	assureCursorVisible();
 }
 
-void PPListBox::setLocation(PPPoint p)
+void PPListBox::setLocation(const PPPoint& p)
 {
 	PPControl::setLocation(p);
 
@@ -1153,7 +1150,7 @@ void PPListBox::addItem(const PPString& item)
 	adjustScrollbars();
 }
 
-const PPString& PPListBox::getItem(pp_int32 index)
+const PPString& PPListBox::getItem(pp_int32 index) const
 {
 	return *items->get(index);
 }
