@@ -1,7 +1,7 @@
 /*
  *  ppui/Slider.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -70,7 +70,9 @@ void PPSlider::initButtons()
 	setBarPosition(currentBarPosition, false);
 }
 
-PPSlider::PPSlider(pp_int32 id, PPScreen* parentScreen, EventListenerInterface* eventListener, PPPoint location, pp_int32 size, bool horizontal, bool buttonSwap) :
+PPSlider::PPSlider(pp_int32 id, PPScreen* parentScreen, EventListenerInterface* eventListener, 
+				   const PPPoint& location, pp_int32 size, 
+				   bool horizontal, bool buttonSwap) :
 	PPControl(id, parentScreen, eventListener, location, PPSize(0,0)),
 	oneDimSize(size),
 	horizontal(horizontal),
@@ -146,7 +148,7 @@ void PPSlider::setBarSize(pp_int32 size, bool repaint /* = true */)
 
 }
 
-void PPSlider::setLocation(PPPoint location)
+void PPSlider::setLocation(const PPPoint& location)
 {
 	PPControl::setLocation(location);
 	
@@ -186,7 +188,7 @@ void PPSlider::setBarPosition(pp_int32 pos, bool repaint /* = false */)
 		parentScreen->paintControl(this);
 }
 
-pp_int32 PPSlider::callEventListener(PPEvent* event)
+pp_int32 PPSlider::dispatchEvent(PPEvent* event)
 {
 	//if (!visible)
 	//	return 0;
@@ -221,13 +223,13 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 			if (buttonUp->hit(*p))
 			{
 				caughtControl = buttonUp;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				controlCaughtByLMouseButton = true;
 			}
 			else if (buttonDown->hit(*p))
 			{
 				caughtControl = buttonDown;
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByLMouseButton = true;
 			}
 			else if (buttonBar->hit(*p))
@@ -239,7 +241,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 				// -------------------------
 				caughtMouseLocation = *p;
 				caughtControlLocation = buttonBar->getLocation();
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByLMouseButton = true;
 			}
 			else if (backgroundButton->hit(*p))
@@ -258,7 +260,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 
 			if (controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 
@@ -270,7 +272,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 			}
 			// -------------------------
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			caughtControl = NULL;
 			break;
 
@@ -278,7 +280,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 		{
 			if (caughtControl)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 			else
@@ -303,7 +305,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 					caughtControl = buttonBar;
 					caughtMouseLocation = *p;
 					caughtControlLocation = buttonBar->getLocation();
-					caughtControl->callEventListener(event);				
+					caughtControl->dispatchEvent(event);				
 				}
 
 			}
@@ -317,13 +319,13 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 			if (buttonUp->hit(*p))
 			{
 				caughtControl = buttonUp;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				controlCaughtByRMouseButton = true;
 			}
 			else if (buttonDown->hit(*p))
 			{
 				caughtControl = buttonDown;
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByRMouseButton = true;
 			}
 			else if (buttonBar->hit(*p))
@@ -335,7 +337,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 				// -------------------------
 				caughtMouseLocation = *p;
 				caughtControlLocation = buttonBar->getLocation();
-				caughtControl->callEventListener(event);				
+				caughtControl->dispatchEvent(event);				
 				controlCaughtByRMouseButton = true;
 			}
 			else if (backgroundButton->hit(*p))
@@ -354,7 +356,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 
 			if (controlCaughtByLMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 
@@ -366,7 +368,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 			}
 			// -------------------------
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			caughtControl = NULL;
 			break;
 
@@ -374,7 +376,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 		{
 			if (caughtControl)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 			else
@@ -399,7 +401,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 					caughtControl = buttonBar;
 					caughtMouseLocation = *p;
 					caughtControlLocation = buttonBar->getLocation();
-					caughtControl->callEventListener(event);				
+					caughtControl->dispatchEvent(event);				
 				}
 
 			}
@@ -411,7 +413,7 @@ pp_int32 PPSlider::callEventListener(PPEvent* event)
 			if (caughtControl == NULL)
 				break;
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			break;
 
 	}
