@@ -1,7 +1,7 @@
 /*
  *  tracker/EnvelopeEditorControl.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -46,8 +46,8 @@
 EnvelopeEditorControl::EnvelopeEditorControl(pp_int32 id, 
 											 PPScreen* parentScreen, 
 											 EventListenerInterface* eventListener, 
-											 PPPoint location, 
-											 PPSize size, 
+											 const PPPoint& location, 
+											 const PPSize& size, 
 											 bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
 	borderColor(&ourOwnBorderColor),
@@ -503,7 +503,7 @@ void EnvelopeEditorControl::paint(PPGraphicsAbstract* g)
 
 }
 
-pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
+pp_int32 EnvelopeEditorControl::dispatchEvent(PPEvent* event)
 { 
 	if (eventListener == NULL)
 		return -1;
@@ -522,7 +522,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 					break;
 				controlCaughtByRMouseButton = true;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			}
 			break;
 		}
@@ -541,7 +541,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 					break;
 				controlCaughtByLMouseButton = true;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			}
 			// Clicked in client area
 			else
@@ -606,7 +606,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 			controlCaughtByRMouseButton = false;
 			if (caughtControl && !controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;			
 				break;
 			}
@@ -624,7 +624,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 
 			if (!controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;
 			}
 			break;
@@ -633,7 +633,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 		{
 			if (caughtControl && controlCaughtByLMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 
@@ -662,7 +662,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 		{
 			if (caughtControl && controlCaughtByLMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}		
 
@@ -687,14 +687,14 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 		case eRMouseDrag:
 		{
 			if (caughtControl && controlCaughtByRMouseButton)
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			break;
 		}
 
 		case eRMouseRepeat:
 		{
 			if (caughtControl && controlCaughtByRMouseButton)
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			break;
 		}
 
@@ -723,7 +723,7 @@ pp_int32 EnvelopeEditorControl::callEventListener(PPEvent* event)
 			if (caughtControl == NULL)
 				break;
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			break;
 
 	}
@@ -788,14 +788,14 @@ pp_int32 EnvelopeEditorControl::handleEvent(PPObject* sender, PPEvent* event)
 	return 0;
 }
 
-void EnvelopeEditorControl::setSize(PPSize size)
+void EnvelopeEditorControl::setSize(const PPSize& size)
 {
 	PPControl::setSize(size);
 
 	hScrollbar->setSize(size.width - 1);
 }
 
-void EnvelopeEditorControl::setLocation(PPPoint location)
+void EnvelopeEditorControl::setLocation(const PPPoint& location)
 {
 	PPControl::setLocation(location);
 
@@ -1043,7 +1043,7 @@ bool EnvelopeEditorControl::invokeTool(EnvelopeToolTypes type)
 		{
 			lastValues.scaleEnvelope = static_cast<DialogWithValues*>(dialog)->getValueOne();
 			FilterParameters par(1);
-			par.setParameter(0, lastValues.scaleEnvelope / 100.0f);
+			par.setParameter(0, FilterParameters::Parameter(lastValues.scaleEnvelope / 100.0f));
 			envelopeEditor->tool_xScaleEnvelope(&par);
 			break;
 		}
@@ -1052,7 +1052,7 @@ bool EnvelopeEditorControl::invokeTool(EnvelopeToolTypes type)
 		{
 			lastValues.scaleEnvelope = static_cast<DialogWithValues*>(dialog)->getValueOne();
 			FilterParameters par(1);
-			par.setParameter(0, lastValues.scaleEnvelope / 100.0f);
+			par.setParameter(0, FilterParameters::Parameter(lastValues.scaleEnvelope / 100.0f));
 			envelopeEditor->tool_yScaleEnvelope(&par);
 			break;
 		}
