@@ -1,7 +1,7 @@
 /*
  *  tracker/PatternEditor.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -69,7 +69,7 @@ void PatternEditor::prepareUndo()
 	before = new PatternUndoStackEntry(*pattern, cursor.channel, cursor.row, cursor.inner, &undoUserData);
 }
 
-bool PatternEditor::finishUndo(LastChanges lastChange)
+bool PatternEditor::finishUndo(LastChanges lastChange, bool nonRepeat/* = false*/)
 {
 	bool result = false;
 
@@ -99,7 +99,10 @@ bool PatternEditor::finishUndo(LastChanges lastChange)
 		notifyListener(NotificationChanges);
 		if (undoStack) 
 		{ 
-			undoStack->Push(*before); 
+			if (nonRepeat && this->lastChange != lastChange)
+				undoStack->Push(*before); 	
+			else if (!nonRepeat)
+				undoStack->Push(*before); 				
 			undoStack->Push(after); 
 			undoStack->Pop(); 
 		} 
@@ -109,47 +112,6 @@ bool PatternEditor::finishUndo(LastChanges lastChange)
 	return result;
 }
 
-bool PatternEditor::finishUndoNonRepeat(LastChanges lastChange) 
-{
-	bool result = false;
-
-	PatternEditorTools patternEditorTools(pattern); 
-	patternEditorTools.normalize(); 
-
-	undoUserData.clear();
-	notifyListener(NotificationFeedUndoData);
-
-	PatternUndoStackEntry after(*pattern, cursor.channel, cursor.row, cursor.inner, &undoUserData); 
-	if (*before != after) 
-	{ 
-		PatternEditorTools::Position afterPos;
-		afterPos.channel = after.getCursorPositionChannel();
-		afterPos.row = after.getCursorPositionRow();
-		afterPos.inner = after.getCursorPositionInner();
-
-		PatternEditorTools::Position beforePos;		
-		beforePos.channel = before->getCursorPositionChannel();
-		beforePos.row = before->getCursorPositionRow();
-		beforePos.inner = before->getCursorPositionInner();
-	
-		result = true;
-
-		lastOperationDidChangeRows = after.GetPattern().rows != before->GetPattern().rows;
-		lastOperationDidChangeCursor = beforePos != afterPos;
-		if (undoStack) 
-		{ 
-			if (this->lastChange != lastChange) 
-				undoStack->Push(*before);  
-			undoStack->Push(after); 
-			undoStack->Pop(); 
-		} 
-		notifyListener(NotificationChanges);
-	} 
-	this->lastChange = lastChange;
-	
-	return result;
-}
-
 PatternEditor::PatternEditor() :
 	EditorBase(),
 	pattern(NULL),
@@ -362,7 +324,9 @@ bool PatternEditor::revoke(const PatternUndoStackEntry* stackEntry)
 
 		// keep over userdata
 		undoUserData = stackEntry->getUserData();
-		notifyListener(NotificationFetchUndoData);		
+		notifyListener(NotificationFetchUndoData);
+
+		notifyListener(NotificationChanges);
 		res = true;
 	}
 	
@@ -520,7 +484,7 @@ bool PatternEditor::resizePattern(pp_int32 newRowNum, bool withUndo /*= true*/)
 		// Special treatment for pattern resizing:
 		// If user resizes pattern and the last stack entry has already been
 		// a resize modification we don't store the current changes		
-		finishUndoNonRepeat(LastChangeResizePattern);		
+		finishUndo(LastChangeResizePattern, true);		
 	}
 	else
 	{
@@ -1525,16 +1489,18 @@ void PatternEditor::storeMacroFromCursor(pp_int32 slot)
 	if (cursor.inner >= 3 && cursor.inner <= 4)
 	{
 		patternTools.getEffect(0, eff, op);
-		if (!eff && !op)
-			return;
+		// feature shall also work with empty effects
+		//if (!eff && !op)
+		//	return;
 		effectMacros[slot].effect = (pp_uint8)eff;
 		effectMacros[slot].operand = (pp_uint8)op;
 	}
 	else if (cursor.inner > 4)
 	{
 		patternTools.getEffect(1, eff, op);
-		if (!eff && !op)
-			return;
+		// feature shall also work with empty effects
+		//if (!eff && !op)
+		//	return;
 		effectMacros[slot+10].effect = (pp_uint8)eff;
 		effectMacros[slot+10].operand = (pp_uint8)op;
 	}
@@ -1555,21 +1521,23 @@ void PatternEditor::writeMacroToCursor(pp_int32 slot, PatternAdvanceInterface* a
 	
 	if (cursor.inner >= 3 && cursor.inner <= 4)
 	{
-		if (!effectMacros[slot].effect && !effectMacros[slot].operand)
-			goto writeNothing;			
+		// feature shall also work with empty effects
+		//if (!effectMacros[slot].effect && !effectMacros[slot].operand)
+		//	goto writeNothing;			
 		patternTools.setEffect(0, effectMacros[slot].effect, effectMacros[slot].operand);
 	}
 	else if (cursor.inner > 4)
 	{
-		if (!effectMacros[slot+10].effect && !effectMacros[slot+10].operand)
-			goto writeNothing;
+		// feature shall also work with empty effects
+		//if (!effectMacros[slot+10].effect && !effectMacros[slot+10].operand)
+		//	goto writeNothing;
 		patternTools.setEffect(1, effectMacros[slot+10].effect, effectMacros[slot+10].operand);
 	}
 
 	if (advanceImpl)
 		advanceImpl->advance();
 
-writeNothing:
+//writeNothing:
 
 	finishUndo(LastChangeWriteMacro);
 }
