@@ -1,7 +1,7 @@
 /*
  *  tracker/PatternEditorControl.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -37,7 +37,8 @@
 
 #define SCROLLBARWIDTH  SCROLLBUTTONSIZE
 
-PatternEditorControl::PatternEditorControl(pp_int32 id, PPScreen* parentScreen, EventListenerInterface* eventListener, PPPoint location, PPSize size, bool border/*= true*/) :
+PatternEditorControl::PatternEditorControl(pp_int32 id, PPScreen* parentScreen, EventListenerInterface* eventListener, 
+										   const PPPoint& location, const PPSize& size, bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
 	borderColor(&TrackerConfig::colorThemeMain),
 	cursorColor(&TrackerConfig::colorPatternEditorCursorLine),
@@ -149,7 +150,7 @@ void PatternEditorControl::setFont(PPFont* font)
 	assureCursorVisible();
 }
 
-void PatternEditorControl::setSize(PPSize size)
+void PatternEditorControl::setSize(const PPSize& size)
 {
 	PPControl::setSize(size);
 	
@@ -170,7 +171,7 @@ void PatternEditorControl::setSize(PPSize size)
 	assureCursorVisible();
 }
 
-void PatternEditorControl::setLocation(PPPoint location)
+void PatternEditorControl::setLocation(const PPPoint& location)
 {
 	PPControl::setLocation(location);
 
@@ -1259,7 +1260,7 @@ void PatternEditorControl::validate()
 		selectionEnd.row = pattern->rows-1;*/
 }
 
-bool PatternEditorControl::hasValidSelection()
+bool PatternEditorControl::hasValidSelection() const
 {
 	return patternEditor->hasValidSelection();
 }
@@ -1390,12 +1391,14 @@ void PatternEditorControl::switchEditMode(EditModes mode)
 		}
 
 		case EditModeFastTracker:
+		{
 			// Assign keyboard bindings
 			eventKeyDownBindings = eventKeyDownBindingsFastTracker;
 			scanCodeBindings = scanCodeBindingsFastTracker;
 
 			selectionKeyModifier = KeyModifierALT;
 			break;
+		}
 	}
 
 	editMode = mode;
@@ -1411,10 +1414,7 @@ void PatternEditorControl::unmuteAll()
 
 void PatternEditorControl::setRecordMode(bool b)
 {
-	if (b)
-		cursorColor = &TrackerConfig::colorPatternEditorCursorLineHighLight;
-	else
-		cursorColor = &TrackerConfig::colorPatternEditorCursorLine;
+	cursorColor = b ? &TrackerConfig::colorPatternEditorCursorLineHighLight : &TrackerConfig::colorPatternEditorCursorLine;
 }
 
 void PatternEditorControl::editorNotification(EditorBase* sender, EditorBase::EditorNotifications notification)
