@@ -1,7 +1,7 @@
 /*
  *  tracker/PianoControl.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -98,17 +98,16 @@ pp_int32 PianoControl::KEYWIDTH()
 PianoControl::PianoControl(pp_int32 id, 
 						   PPScreen* parentScreen, 
 						   EventListenerInterface* eventListener, 
-						   PPPoint location, 
-						   PPSize size, 
+						   const PPPoint& location, 
+						   const PPSize& size, 
 						   pp_uint8 numNotes,
 						   bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
+	border(border),
 	NUMNOTES(numNotes),
 	borderColor(&ourOwnBorderColor),
 	mode(ModeEdit)
 {
-	this->border = border;
-
 	// default color
 	ourOwnBorderColor.set(192, 192, 192);
 
@@ -364,7 +363,7 @@ void PianoControl::paint(PPGraphicsAbstract* g)
 
 }
 
-pp_int32 PianoControl::callEventListener(PPEvent* event)
+pp_int32 PianoControl::dispatchEvent(PPEvent* event)
 { 
 	if (eventListener == NULL)
 		return -1;
@@ -384,7 +383,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 					break;
 				controlCaughtByRMouseButton = true;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			}
 			break;
 		}		
@@ -400,7 +399,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 					break;
 				controlCaughtByLMouseButton = true;
 				caughtControl = hScrollbar;
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			}
 			// Clicked in client area
 			else
@@ -442,7 +441,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 			controlCaughtByRMouseButton = false;
 			if (caughtControl && !controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;			
 				break;
 			}
@@ -476,7 +475,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 			controlCaughtByLMouseButton = false;
 			if (!controlCaughtByLMouseButton && !controlCaughtByRMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				caughtControl = NULL;
 			}
 			break;
@@ -485,7 +484,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 		{
 			if (caughtControl && controlCaughtByLMouseButton)
 			{
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 				break;
 			}
 			
@@ -535,14 +534,14 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 		case eRMouseDrag:
 		{
 			if (caughtControl && controlCaughtByRMouseButton)
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			break;
 		}
 		
 		case eRMouseRepeat:
 		{
 			if (caughtControl && controlCaughtByRMouseButton)
-				caughtControl->callEventListener(event);
+				caughtControl->dispatchEvent(event);
 			break;
 		}
 		
@@ -570,7 +569,7 @@ pp_int32 PianoControl::callEventListener(PPEvent* event)
 			if (caughtControl == NULL)
 				break;
 
-			caughtControl->callEventListener(event);
+			caughtControl->dispatchEvent(event);
 			break;
 
 	}
@@ -675,7 +674,7 @@ void PianoControl::setyScale(pp_int32 scale)
 	yscale = scale; 
 }
 
-void PianoControl::setLocation(PPPoint location)
+void PianoControl::setLocation(const PPPoint& location)
 {
 	PPControl::setLocation(location);
 
@@ -687,7 +686,7 @@ void PianoControl::setLocation(PPPoint location)
 	hScrollbar->setBarPosition(p);
 }
 
-void PianoControl::setSize(PPSize size)
+void PianoControl::setSize(const PPSize& size)
 {
 	PPControl::setSize(size);
 	delete hScrollbar;
@@ -721,7 +720,7 @@ void PianoControl::pressNote(pp_int32 note, bool pressed, bool muted/* = false*/
 	}
 }
 
-bool PianoControl::getNoteState(pp_int32 note)
+bool PianoControl::getNoteState(pp_int32 note) const
 {
 	if (note >= 0 && note < NUMNOTES)
 		return keyState[note].pressed;
