@@ -1,7 +1,7 @@
 /*
  *  tracker/ScopesControl.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -63,12 +63,12 @@ pp_int32 ScopesControl::WRAPCHANNELS()
 }
 
 ScopesControl::ScopesControl(pp_int32 id, 
-								   PPScreen* parentScreen, 
-								   EventListenerInterface* eventListener, 
-								   PPPoint location, 
-								   PPSize size, 
-								   bool border/*= true*/) :
+							 PPScreen* parentScreen, 
+							 EventListenerInterface* eventListener, 
+							 const PPPoint& location, const PPSize& size, 
+							 bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
+	border(border),
 	borderColor(&ourOwnBorderColor),
 	playerController(NULL),
 	numChannels(0),
@@ -76,8 +76,6 @@ ScopesControl::ScopesControl(pp_int32 id,
 	appearance(AppearanceTypeNormal),
 	currentClickType(ClickTypeMute)
 {
-	this->border = border;
-
 	// default color
 	color.set(0, 0, 0);
 
@@ -545,7 +543,15 @@ void ScopesControl::handleSingleRec(pp_int32 channel)
 	eventListener->handleEvent(reinterpret_cast<PPObject*>(this), &e);
 }
 
-pp_int32 ScopesControl::callEventListener(PPEvent* event)
+void ScopesControl::handleUnmuteAll()
+{
+	for (pp_uint32 i = 0; i < sizeof(muteChannels)/sizeof(pp_uint8); i++)
+		muteChannels[i] = false;
+	PPEvent e(eValueChanged, &muteChannels, sizeof(muteChannels), ChangeValueMuting);						
+	eventListener->handleEvent(reinterpret_cast<PPObject*>(this), &e);
+}
+
+pp_int32 ScopesControl::dispatchEvent(PPEvent* event)
 { 
 	if (eventListener == NULL)
 		return -1;
@@ -620,7 +626,7 @@ pp_int32 ScopesControl::callEventListener(PPEvent* event)
 	return 0;
 }
 
-bool ScopesControl::isSoloChannel(pp_int32 c)
+bool ScopesControl::isSoloChannel(pp_int32 c) const
 {
 	pp_int32 i = 0;
 	for (pp_int32 j = 0; j < numChannels; j++)
@@ -633,7 +639,7 @@ bool ScopesControl::isSoloChannel(pp_int32 c)
 	return false;
 }
 
-bool ScopesControl::isSingleRecChannel(pp_int32 c)
+bool ScopesControl::isSingleRecChannel(pp_int32 c) const
 {
 	pp_int32 i = 0;
 	for (pp_int32 j = 0; j < numChannels; j++)
