@@ -1,7 +1,7 @@
 /*
  *  tracker/SectionHDRecorder.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -852,7 +852,7 @@ void SectionHDRecorder::exportWAVAs(const PPSystemString& fileName)
 {
 	if (tracker.useClassicBrowser)
 	{
-		tracker.saveType(Tracker::FileTypeSongWAV);
+		tracker.saveType(FileTypes::FileTypeSongWAV);
 	}
 	else
 	{
