@@ -1,7 +1,7 @@
 /*
  *  tracker/SectionInstruments.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -392,7 +392,7 @@ pp_int32 SectionInstruments::handleEvent(PPObject* sender, PPEvent* event)
 				if (event->getID() != eCommand)
 					break;
 
-				tracker.loadType(Tracker::FileTypeSongAllInstruments);
+				tracker.loadType(FileTypes::FileTypeSongAllInstruments);
 				break;
 			}
 
@@ -402,7 +402,7 @@ pp_int32 SectionInstruments::handleEvent(PPObject* sender, PPEvent* event)
 				if (event->getID() != eCommand)
 					break;
 
-				tracker.saveType(Tracker::FileTypeInstrumentXI);
+				tracker.saveType(FileTypes::FileTypeInstrumentXI);
 				break;
 			}
 
@@ -537,7 +537,7 @@ pp_int32 SectionInstruments::handleEvent(PPObject* sender, PPEvent* event)
 			{
 				tracker.setChanged();
 				pp_int32 i = tracker.listBoxInstruments->getSelectedIndex();	
-				moduleEditor->updateSampleTable(i, *(reinterpret_cast<pp_uint8**>(event->getDataPtr())));
+				moduleEditor->updateSampleTable(i, *(reinterpret_cast<const pp_uint8* const*>(event->getDataPtr())));
 				break;
 			}		
 		}
