@@ -1,7 +1,7 @@
 /*
  *  tracker/Tracker.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -44,6 +44,7 @@
 #include "KeyBindings.h"
 #include "ModuleServices.h"
 #include "FileIdentificator.h"
+#include "FileExtProvider.h"
 #include "Decompressor.h"
 #include "Zapper.h"
 #include "TitlePageManager.h"
@@ -69,8 +70,6 @@
 #include "DialogZap.h"
 // Helper class to invoke tools which need parameters
 #include "ToolInvokeHelper.h"
-// Panning settings container modal dialog
-#include "PanningDialog.h"
 
 #include "ControlIDs.h"
 
@@ -253,6 +252,10 @@ pp_int32 Tracker::getOrderListBoxIndex()
 
 void Tracker::setOrderListIndex(pp_int32 index)
 {
+	// do not accept values which exceed the current number of orders
+	if (index >= moduleEditor->getNumOrders())
+		index = moduleEditor->getNumOrders()-1;
+		
 	listBoxOrderList->setSelectedIndex(index);
 	updateOrderlist();
 	// fake selection from orderlist, so everything will be updated correctly
@@ -526,7 +529,7 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 	{
 		if (screen->getModalControl())
 			return 0;
-		PPSystemString* str = *(reinterpret_cast<PPSystemString**>(event->getDataPtr()));
+		const PPSystemString* str = *(reinterpret_cast<PPSystemString* const*>(event->getDataPtr()));
 		loadGenericFileType(*str);
 		event->cancel();
 	}
@@ -1272,9 +1275,7 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 			case LISTBOX_INSTRUMENTS:
 			{
 				pp_int32 index = *((pp_int32*)event->getDataPtr()) + 1;
-				selectInstrument(index);
-				for (pp_int32 i = 0; i < sections->size(); i++)
-					sections->get(i)->notifyInstrumentSelect(index);
+				selectInstrument(index);		
 				screen->update();
 				break;
 			}
@@ -1438,20 +1439,20 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 		{
 			case LISTBOX_SONGTITLE:
 			{
-				moduleEditor->setTitle(**(reinterpret_cast<PPString**>(event->getDataPtr())), ModuleEditor::MAX_TITLETEXT);
+				moduleEditor->setTitle(**(reinterpret_cast<const PPString* const*>(event->getDataPtr())), ModuleEditor::MAX_TITLETEXT);
 				break;
 			}
 		
 			case LISTBOX_INSTRUMENTS:
 			{
 				moduleEditor->setInstrumentName(listBoxInstruments->getSelectedIndex(), 
-												**(reinterpret_cast<PPString**>(event->getDataPtr())), ModuleEditor::MAX_INSTEXT);
+												**(reinterpret_cast<const PPString* const*>(event->getDataPtr())), ModuleEditor::MAX_INSTEXT);
 				break;
 			}
 			
 			case LISTBOX_SAMPLES:
 			{
-				moduleEditor->setCurrentSampleName(**(reinterpret_cast<PPString**>(event->getDataPtr())), 
+				moduleEditor->setCurrentSampleName(**(reinterpret_cast<const PPString* const*>(event->getDataPtr())), 
 												   ModuleEditor::MAX_SMPTEXT);
 				break;
 			}
@@ -1459,7 +1460,7 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 			// channels have been muted/unmuted in pattern editor
 			case PATTERN_EDITOR:
 			{
-				const pp_uint8* muteChannelsPtr = reinterpret_cast<pp_uint8*>(event->getDataPtr());				
+				const pp_uint8* muteChannelsPtr = reinterpret_cast<const pp_uint8*>(event->getDataPtr());				
 				
 				for (pp_int32 i = 0; i < TrackerConfig::numPlayerChannels; i++)
 				{
@@ -1481,7 +1482,7 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 				{
 					case ScopesControl::ChangeValueMuting:
 					{
-						const pp_uint8* muteChannelsPtr = reinterpret_cast<pp_uint8*>(event->getDataPtr());				
+						const pp_uint8* muteChannelsPtr = reinterpret_cast<const pp_uint8*>(event->getDataPtr());				
 						
 						for (pp_int32 i = 0; i < TrackerConfig::numPlayerChannels; i++)
 						{
@@ -1501,7 +1502,7 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 					
 					case ScopesControl::ChangeValueRecording:
 					{
-						const pp_uint8* recordChannelsPtr = reinterpret_cast<pp_uint8*>(event->getDataPtr());				
+						const pp_uint8* recordChannelsPtr = reinterpret_cast<const pp_uint8*>(event->getDataPtr());				
 						
 						for (pp_int32 i = 0; i < TrackerConfig::numPlayerChannels; i++)
 						{
@@ -1993,29 +1994,7 @@ bool Tracker::messageBoxEventListener(pp_int32 messageBoxID, pp_int32 messageBox
 			}
 
 			break;
-		}
-		
-		case MESSAGEBOX_SAVEPROCEED:
-		{
-			switch (messageBoxButtonID)
-			{
-				case PP_MESSAGEBOX_BUTTON_YES:
-				{
-					if (savePanel != NULL)
-						saveTypeWithDialog(currentSaveFileType, fileSystemChangedListener);
-					else
-						saveCurrentModuleAsSelectedType();
-					break;
-				}
-				
-				case PP_MESSAGEBOX_BUTTON_NO:
-					delete savePanel;
-					savePanel = NULL;
-					break;
-			}
-			break;
-		}
-
+		}		
 	}
 	
 	return true;
@@ -2160,6 +2139,9 @@ void Tracker::selectInstrument(pp_int32 instrument)
 	updateSampleEditor(false);
 	// update instrument/sample editor
 	sectionInstruments->update(false);
+
+	for (pp_int32 i = 0; i < sections->size(); i++)
+		sections->get(i)->notifyInstrumentSelect(instrument);
 }
 
 void Tracker::fillInstrumentListBox(PPListBox* listBox, ModuleEditor* moduleEditor/* = NULL*/)
@@ -2497,11 +2479,59 @@ void Tracker::commitListBoxChanges()
 		listBoxSamples->commitChanges();
 }
 
-Tracker::FileTypes Tracker::getCurrentSelectedSampleSaveType()
+FileTypes Tracker::getCurrentSelectedSampleSaveType()
 {
 	return (FileTypes)sectionDiskMenu->getCurrentSelectedSampleSaveType();
 }
 
+pp_uint32 Tracker::fileTypeToHint(FileTypes type)
+{
+	switch (type)
+	{
+		case FileTypes::FileTypeSongAllModules:			
+		case FileTypes::FileTypeSongMOD:
+		case FileTypes::FileTypeSongXM:
+			return DecompressorBase::HintModules;
+
+		case FileTypes::FileTypePatternXP:
+			return DecompressorBase::HintPatterns;
+
+		case FileTypes::FileTypeTrackXT:
+			return DecompressorBase::HintTracks;
+
+		case FileTypes::FileTypeSongAllInstruments:
+		case FileTypes::FileTypeInstrumentXI:
+			return DecompressorBase::HintInstruments;
+
+		case FileTypes::FileTypeSongAllSamples:
+		case FileTypes::FileTypeSampleWAV:
+		case FileTypes::FileTypeSampleIFF:
+			return DecompressorBase::HintSamples;
+
+		default:
+			return DecompressorBase::HintAll;
+
+	}
+}
+
+void Tracker::prepareLoadSaveUI()
+{
+#ifdef __LOWRES__
+		// The bottom section fills up the entire screen 
+		// so we first need to hide the entire section before we can show the disk menu
+		screen->pauseUpdate(true);
+		sectionSwitcher->hideBottomSection();
+#endif
+}
+
+void Tracker::finishLoadSaveUI()
+{
+#ifdef __LOWRES__
+		screen->pauseUpdate(false);
+		screen->paint();
+#endif
+}
+
 bool Tracker::loadGenericFileType(const PPSystemString& fileName)
 {
 	// we need to find out what file type this is
@@ -2516,7 +2546,7 @@ bool Tracker::loadGenericFileType(const PPSystemString& fileName)
 		// and choose that file type
 		PPSystemString tempFile(ModuleEditor::getTempFilename());
 		Decompressor decompressor(fileName);
-		if (decompressor.decompress(tempFile))
+		if (decompressor.decompress(tempFile, (DecompressorBase::Hints)fileTypeToHint(FileTypes::FileTypeAllFiles)))
 		{
 			fileIdentificator = new FileIdentificator(tempFile);
 			type = fileIdentificator->getFileType();
@@ -2533,15 +2563,15 @@ bool Tracker::loadGenericFileType(const PPSystemString& fileName)
 	switch (type)
 	{
 		case FileIdentificator::FileTypeModule:
-			return loadTypeFromFile(FileTypeSongAllModules, fileName);		
+			return loadTypeFromFile(FileTypes::FileTypeSongAllModules, fileName);		
 		case FileIdentificator::FileTypeInstrument:
-			return loadTypeFromFile(FileTypeSongAllInstruments, fileName);
+			return loadTypeFromFile(FileTypes::FileTypeSongAllInstruments, fileName);
 		case FileIdentificator::FileTypeSample:
-			return loadTypeFromFile(FileTypeSongAllSamples, fileName);
+			return loadTypeFromFile(FileTypes::FileTypeSongAllSamples, fileName);
 		case FileIdentificator::FileTypePattern:
-			return loadTypeFromFile(FileTypePatternXP, fileName);
+			return loadTypeFromFile(FileTypes::FileTypePatternXP, fileName);
 		case FileIdentificator::FileTypeTrack:
-			return loadTypeFromFile(FileTypeTrackXT, fileName);
+			return loadTypeFromFile(FileTypes::FileTypeTrackXT, fileName);
 	}
 	
 	return false;
@@ -2559,14 +2589,14 @@ bool Tracker::prepareLoading(FileTypes eType, const PPSystemString& fileName, bo
 
 	loadingParameters.res = true;
 	
-	if (saveCheck && eType == FileTypeSongAllModules && !checkForChangesOpenModule())
+	if (saveCheck && eType == FileTypes::FileTypeSongAllModules && !checkForChangesOpenModule())
 		return false;
 	
 	loadingParameters.lastError = "Error while loading/unknown format";	
 
 	signalWaitState(true);
 
-	if (eType == FileTypeSongAllModules &&
+	if (eType == FileTypes::FileTypeSongAllModules &&
 		settingsDatabase->restore("TABS_LOADMODULEINNEWTAB")->getBoolValue() &&
 		(moduleEditor->hasChanged() || !moduleEditor->isEmpty()))
 	{
@@ -2596,7 +2626,7 @@ bool Tracker::prepareLoading(FileTypes eType, const PPSystemString& fileName, bo
 		// if this is compressed, try to decompress
 		PPSystemString tempFile(ModuleEditor::getTempFilename());
 		Decompressor decompressor(fileName);
-		if (decompressor.decompress(tempFile))
+		if (decompressor.decompress(tempFile, (DecompressorBase::Hints)fileTypeToHint(eType)))
 		{
 			// we compressed to a temporary file
 			// load that instead, but keep the original file name as preferred 
@@ -2654,7 +2684,7 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 	
 	switch (eType)
 	{
-		case FileTypeSongAllModules:
+		case FileTypes::FileTypeSongAllModules:
 		{
 			if (loadingParameters.preferredFilename.length())
 				loadingParameters.res = moduleEditor->openSong(loadingParameters.filename,
@@ -2667,28 +2697,28 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 			break;
 		}
 			
-		case FileTypePatternXP:
+		case FileTypes::FileTypePatternXP:
 		{
 			loadingParameters.res = getPatternEditor()->loadExtendedPattern(loadingParameters.filename);
 			updateSongInfo();
 			break;
 		}
 			
-		case FileTypeTrackXT:
+		case FileTypes::FileTypeTrackXT:
 		{
 			loadingParameters.res = getPatternEditor()->loadExtendedTrack(loadingParameters.filename);
 			updateSongInfo();
 			break;
 		}
 			
-		case FileTypeSongAllInstruments:
+		case FileTypes::FileTypeSongAllInstruments:
 		{
 			loadingParameters.res = moduleEditor->loadInstrument(loadingParameters.filename, listBoxInstruments->getSelectedIndex());
 			sectionInstruments->updateAfterLoad();
 			break;
 		}
 			
-		case FileTypeSongAllSamples:
+		case FileTypes::FileTypeSongAllSamples:
 		{
 			pp_int32 numSampleChannels = moduleEditor->getNumSampleChannels(loadingParameters.filename);
 			
@@ -2753,44 +2783,46 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 // load different things
 bool Tracker::loadTypeWithDialog(FileTypes eLoadType, bool suspendPlayer/* = true*/, bool repaint/* = true*/)
 {
+	FileExtProvider fileExtProvider;
+
 	PPOpenPanel* openPanel = NULL;
 
 	switch (eLoadType)
 	{
-		case FileTypeSongAllModules:
+		case FileTypes::FileTypeSongAllModules:
 		{
 			if (!checkForChangesOpenModule())
 				return false;
 			openPanel = new PPOpenPanel(screen, "Open module");		
-			openPanel->addExtensions(TrackerConfig::moduleExtensions);
+			openPanel->addExtensions(fileExtProvider.getModuleExtensions());
 			break;
 		}
 	
-		case FileTypePatternXP:
+		case FileTypes::FileTypePatternXP:
 		{
 			openPanel = new PPOpenPanel(screen, "Open Extended Pattern");
-			openPanel->addExtensions(TrackerConfig::patternExtensions);
+			openPanel->addExtensions(fileExtProvider.getPatternExtensions());
 			break;			
 		}
 
-		case FileTypeTrackXT:
+		case FileTypes::FileTypeTrackXT:
 		{
 			openPanel = new PPOpenPanel(screen, "Open Extended Track");
-			openPanel->addExtensions(TrackerConfig::trackExtensions);
+			openPanel->addExtensions(fileExtProvider.getTrackExtensions());
 			break;			
 		}
 
-		case FileTypeSongAllInstruments:
+		case FileTypes::FileTypeSongAllInstruments:
 		{
 			openPanel = new PPOpenPanel(screen, "Open Instrument");
-			openPanel->addExtensions(TrackerConfig::instrumentExtensions);
+			openPanel->addExtensions(fileExtProvider.getInstrumentExtensions());
 			break;
 		}
 
-		case FileTypeSongAllSamples:
+		case FileTypes::FileTypeSongAllSamples:
 		{
 			openPanel = new PPOpenPanel(screen, "Open Sample");
-			openPanel->addExtensions(TrackerConfig::sampleExtensions);
+			openPanel->addExtensions(fileExtProvider.getSampleExtensions());
 			break;
 		}
 	}
@@ -2820,18 +2852,12 @@ void Tracker::loadType(FileTypes eType)
 {
 	if (useClassicBrowser)
 	{
-#ifdef __LOWRES__
-		// The bottom section fills up the entire screen 
-		// so we first need to hide the entire section before we can show the disk menu
-		screen->pauseUpdate(true);
-		sectionSwitcher->hideBottomSection();
-#endif
+		prepareLoadSaveUI();
+
 		sectionDiskMenu->selectSaveType(eType);
 		eventKeyDownBinding_InvokeSectionDiskMenu();
-#ifdef __LOWRES__
-		screen->pauseUpdate(false);
-		screen->paint();
-#endif
+
+		finishLoadSaveUI();
 	}
 	else
 	{
@@ -2841,15 +2867,17 @@ void Tracker::loadType(FileTypes eType)
 
 bool Tracker::prepareSavingWithDialog(FileTypes eSaveType)
 {
+	FileExtProvider fileExtProvider;
+
 	currentSaveFileType = eSaveType;
 	
 	switch (eSaveType)
 	{
-		case FileTypeSongMOD:
+		case FileTypes::FileTypeSongMOD:
 		{
 			savePanel = new PPSavePanel(screen, "Save Protracker Module", moduleEditor->getModuleFileName(ModuleEditor::ModSaveTypeMOD));
-			savePanel->addExtension(TrackerConfig::getModuleExtension(TrackerConfig::ModuleExtensionMOD),
-									TrackerConfig::getModuleDescription(TrackerConfig::ModuleExtensionMOD));
+			savePanel->addExtension(fileExtProvider.getModuleExtension(FileExtProvider::ModuleExtensionMOD),
+									fileExtProvider.getModuleDescription(FileExtProvider::ModuleExtensionMOD));
 			
 			pp_uint32 err = moduleEditor->getPTIncompatibilityCode();
 			
@@ -2861,59 +2889,59 @@ bool Tracker::prepareSavingWithDialog(FileTypes eSaveType)
 			break;
 		}
 			
-		case FileTypeSongXM:
+		case FileTypes::FileTypeSongXM:
 			savePanel = new PPSavePanel(screen, "Save Extended Module", moduleEditor->getModuleFileName(ModuleEditor::ModSaveTypeXM));
-			savePanel->addExtension(TrackerConfig::getModuleExtension(TrackerConfig::ModuleExtensionXM),
-									TrackerConfig::getModuleDescription(TrackerConfig::ModuleExtensionXM));
+			savePanel->addExtension(fileExtProvider.getModuleExtension(FileExtProvider::ModuleExtensionXM),
+									fileExtProvider.getModuleDescription(FileExtProvider::ModuleExtensionXM));
 			break;
 			
-		case FileTypePatternXP:
+		case FileTypes::FileTypePatternXP:
 		{
 			PPSystemString fileName = moduleEditor->getModuleFileName().stripExtension();
 			fileName.append(".xp");
 			savePanel = new PPSavePanel(screen, "Save Extended Pattern", fileName);
-			savePanel->addExtension(TrackerConfig::getPatternExtension(TrackerConfig::PatternExtensionXP),
-									TrackerConfig::getPatternDescription(TrackerConfig::PatternExtensionXP));
+			savePanel->addExtension(fileExtProvider.getPatternExtension(FileExtProvider::PatternExtensionXP),
+									fileExtProvider.getPatternDescription(FileExtProvider::PatternExtensionXP));
 			break;			
 		}
 			
-		case FileTypeTrackXT:
+		case FileTypes::FileTypeTrackXT:
 		{
 			PPSystemString fileName = moduleEditor->getModuleFileName().stripExtension();
 			fileName.append(".xt");
 			savePanel = new PPSavePanel(screen, "Save Extended Track", fileName);
-			savePanel->addExtension(TrackerConfig::getTrackExtension(TrackerConfig::TrackExtensionXT),
-									TrackerConfig::getTrackDescription(TrackerConfig::TrackExtensionXT));
+			savePanel->addExtension(fileExtProvider.getTrackExtension(FileExtProvider::TrackExtensionXT),
+									fileExtProvider.getTrackDescription(FileExtProvider::TrackExtensionXT));
 			break;			
 		}
 			
-		case FileTypeInstrumentXI:
+		case FileTypes::FileTypeInstrumentXI:
 		{
 			PPSystemString sampleFileName = moduleEditor->getInstrumentFileName(listBoxInstruments->getSelectedIndex());
 			sampleFileName.append(".xi");
 			savePanel = new PPSavePanel(screen, "Save Instrument", sampleFileName);
-			savePanel->addExtension(TrackerConfig::getInstrumentExtension(TrackerConfig::InstrumentExtensionXI),
-									TrackerConfig::getInstrumentDescription(TrackerConfig::InstrumentExtensionXI));
+			savePanel->addExtension(fileExtProvider.getInstrumentExtension(FileExtProvider::InstrumentExtensionXI),
+									fileExtProvider.getInstrumentDescription(FileExtProvider::InstrumentExtensionXI));
 			break;
 		}
 			
-		case FileTypeSampleWAV:
+		case FileTypes::FileTypeSampleWAV:
 		{
 			PPSystemString sampleFileName = moduleEditor->getSampleFileName(listBoxInstruments->getSelectedIndex(), listBoxSamples->getSelectedIndex());
 			sampleFileName.append(".wav");
 			savePanel = new PPSavePanel(screen, "Save uncompressed WAV",sampleFileName);
-			savePanel->addExtension(TrackerConfig::getSampleExtension(TrackerConfig::SampleExtensionWAV),
-									TrackerConfig::getSampleDescription(TrackerConfig::SampleExtensionWAV));
+			savePanel->addExtension(fileExtProvider.getSampleExtension(FileExtProvider::SampleExtensionWAV),
+									fileExtProvider.getSampleDescription(FileExtProvider::SampleExtensionWAV));
 			break;
 		}
 			
-		case FileTypeSampleIFF:
+		case FileTypes::FileTypeSampleIFF:
 		{
 			PPSystemString sampleFileName = moduleEditor->getSampleFileName(listBoxInstruments->getSelectedIndex(), listBoxSamples->getSelectedIndex());
 			sampleFileName.append(".iff");
 			savePanel = new PPSavePanel(screen, "Save uncompressed IFF",sampleFileName);
-			savePanel->addExtension(TrackerConfig::getSampleExtension(TrackerConfig::SampleExtensionIFF),
-									TrackerConfig::getSampleDescription(TrackerConfig::SampleExtensionIFF));
+			savePanel->addExtension(fileExtProvider.getSampleExtension(FileExtProvider::SampleExtensionIFF),
+									fileExtProvider.getSampleDescription(FileExtProvider::SampleExtensionIFF));
 			break;
 		}
 	}
@@ -2950,40 +2978,40 @@ bool Tracker::saveTypeWithDialog(FileTypes eSaveType, EventListenerInterface* fi
 			
 			switch (eSaveType)
 			{
-				case FileTypeSongMOD:
+				case FileTypes::FileTypeSongMOD:
 					commitListBoxChanges();
 					res = moduleEditor->saveSong(file, ModuleEditor::ModSaveTypeMOD);
 					break;
 					
-				case FileTypeSongXM:
+				case FileTypes::FileTypeSongXM:
 					commitListBoxChanges();
 					res = moduleEditor->saveSong(file, ModuleEditor::ModSaveTypeXM);
 					break;
 					
-				case FileTypeTrackXT:
+				case FileTypes::FileTypeTrackXT:
 				{
 					res = getPatternEditor()->saveExtendedTrack(file);
 					break;
 				}
 					
-				case FileTypePatternXP:
+				case FileTypes::FileTypePatternXP:
 				{
 					res = getPatternEditor()->saveExtendedPattern(file);
 					break;
 				}
 					
-				case FileTypeInstrumentXI:
+				case FileTypes::FileTypeInstrumentXI:
 					commitListBoxChanges();
 					res = moduleEditor->saveInstrument(file, listBoxInstruments->getSelectedIndex());
 					break;
 					
-				case FileTypeSampleWAV:
+				case FileTypes::FileTypeSampleWAV:
 					commitListBoxChanges();
 					res = moduleEditor->saveSample(file, listBoxInstruments->getSelectedIndex(), 
 												   listBoxSamples->getSelectedIndex(), ModuleEditor::SampleFormatTypeWAV);
 					break;
 					
-				case FileTypeSampleIFF:
+				case FileTypes::FileTypeSampleIFF:
 					commitListBoxChanges();
 					res = moduleEditor->saveSample(file, listBoxInstruments->getSelectedIndex(), 
 												   listBoxSamples->getSelectedIndex(), ModuleEditor::SampleFormatTypeIFF);
@@ -3037,15 +3065,19 @@ void Tracker::saveType(FileTypes eType)
 {
 	if (useClassicBrowser)
 	{
+		prepareLoadSaveUI();
+	
 		sectionDiskMenu->selectSaveType(eType);
 		
-		if (eType == FileTypeSongWAV)
+		if (eType == FileTypes::FileTypeSongWAV)
 			sectionDiskMenu->setModuleTypeAdjust(false);
 		
 		eventKeyDownBinding_InvokeSectionDiskMenu();
 		
-		if (eType == FileTypeSongWAV)
+		if (eType == FileTypes::FileTypeSongWAV)
 			sectionDiskMenu->setModuleTypeAdjust(true);
+	
+		finishLoadSaveUI();
 	}
 	else
 	{
@@ -3053,6 +3085,71 @@ void Tracker::saveType(FileTypes eType)
 	}
 }
 
+void Tracker::save()
+{
+	if (TrackerConfig::untitledSong.compareTo(moduleEditor->getModuleFileNameFull().stripExtension()) == 0)
+	{
+		saveAs();
+	}
+	else
+	{
+		if (moduleEditor->getSaveType() == ModuleEditor::ModSaveTypeMOD)
+		{
+			pp_uint32 err = moduleEditor->getPTIncompatibilityCode();
+			
+			if (err)
+			{
+				// remove save panel
+				// => the handler for the upcoming modal dialog
+				// will note from the absent save panel that saving will be
+				// done to the current document
+				if (savePanel)
+				{
+					delete savePanel;
+					savePanel = NULL;
+				}
+				buildMODSaveErrorWarning((pp_int32)err);
+				return;
+			}
+		}				
+		saveCurrentModuleAsSelectedType();
+	}
+}
+	
+void Tracker::saveAs()
+{
+	switch (moduleEditor->getSaveType())
+	{
+		case ModuleEditor::ModSaveTypeMOD:
+			saveType(FileTypes::FileTypeSongMOD);
+			break;
+
+		case ModuleEditor::ModSaveTypeXM:
+			saveType(FileTypes::FileTypeSongXM);
+			break;
+			
+		default:
+			ASSERT(false);
+	}
+}
+
+void Tracker::handleSaveProceed()
+{
+	// if there is a save panel present we save with file name
+	// selection, otherwise we simply save the current document
+	// in the current format
+	if (savePanel != NULL)
+		saveTypeWithDialog(currentSaveFileType, fileSystemChangedListener);
+	else
+		saveCurrentModuleAsSelectedType();
+}
+
+void Tracker::handleSaveCancel()
+{
+	delete savePanel;
+	savePanel = NULL;
+}
+
 void Tracker::buildMODSaveErrorWarning(pp_int32 error)
 {
 	static const char* warnings[] = {"Song contains more than 31 instruments\nSave anyway?",
@@ -3080,8 +3177,29 @@ void Tracker::buildMODSaveErrorWarning(pp_int32 error)
 									 "* Volume column is not used        \n"\
 									 "* Only effects between 0 and F     \n\nSave anyway?"};
 	
+	if (dialog)
+		delete dialog;
 	
-	showMessageBoxSized(MESSAGEBOX_SAVEPROCEED, warnings[error-1], MessageBox_YESNO, 318);	
+	if (responder)
+		delete responder;
+	
+	responder = new SaveProceedHandler(*this);				
+				
+	dialog = new PPDialogBase(screen, responder, MESSAGEBOX_SAVEPROCEED, "");
+	
+	showMessageBoxSized(MESSAGEBOX_SAVEPROCEED, warnings[error-1], MessageBox_YESNO, 318, -1, false);	
+	
+	// Transfer ownership of modal overlayed container to dialog
+	// => also set new event listener
+	dialog->setMessageBoxContainer(messageBoxContainerGeneric);
+	messageBoxContainerGeneric->setEventListener(dialog);
+	PPSimpleVector<PPControl>& controls = messageBoxContainerGeneric->getControls();
+	for (pp_int32 i = 0; i < controls.size(); i++)
+		controls.get(i)->setEventListener(dialog);
+	
+	messageBoxContainerGeneric = NULL;
+	
+	dialog->show();
 }
 
 void Tracker::estimateSongLength(bool signalWait/* = false*/)
