@@ -1,7 +1,7 @@
 /*
  *  tracker/Tracker.h
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -26,6 +26,7 @@
 #include "BasicTypes.h"
 #include "Event.h"
 #include "EditModes.h"
+#include "FileTypes.h"
 
 #define INPUTCONTAINERHEIGHT_DEFAULT	(25+SCROLLBUTTONSIZE+4)
 #define INPUTCONTAINERHEIGHT_EXTENDED	(25+SCROLLBUTTONSIZE+4+13)
@@ -360,22 +361,7 @@ private:
 	// Signal waitstate
 	void signalWaitState(bool b);
 	
-	// - Save panels -----------------------------------------------------------
-	enum FileTypes
-	{
-		FileTypeSongAllModules,
-		FileTypeSongMOD,
-		FileTypeSongXM,
-		FileTypeSongWAV,
-		FileTypePatternXP,
-		FileTypeTrackXT,
-		FileTypeSongAllInstruments,
-		FileTypeInstrumentXI,
-		FileTypeSongAllSamples,
-		FileTypeSampleWAV,
-		FileTypeSampleIFF
-	};
-
+	// - Load/Save -------------------------------------------------------------
 	FileTypes currentSaveFileType;
 	PPSavePanel* savePanel;
 	EventListenerInterface* fileSystemChangedListener;
@@ -405,6 +391,11 @@ private:
 		}
 	} loadingParameters;
 
+	static pp_uint32 fileTypeToHint(FileTypes type);
+
+	void prepareLoadSaveUI();
+	void finishLoadSaveUI();
+
 	bool loadGenericFileType(const PPSystemString& fileName);
 
 	bool prepareLoading(FileTypes eType, 
@@ -430,6 +421,11 @@ private:
 	bool saveTypeWithDialog(FileTypes eType, EventListenerInterface* fileSystemChangedListener = NULL);
 	bool saveCurrentModuleAsSelectedType();
 	void saveType(FileTypes eType);
+	void save();
+	void saveAs();
+	void handleSaveProceed();
+	void handleSaveCancel();
+	
 	void buildMODSaveErrorWarning(pp_int32 error);
 	void estimateSongLength(bool signalWait = false);
 
@@ -449,6 +445,7 @@ public:
 	
 	PPSize getWindowSizeFromDatabase();
 	bool getFullScreenFlagFromDatabase();
+	pp_int32 getScreenScaleFactorFromDatabase();
 	bool getShowSplashFlagFromDatabase();
 
 	void showSplash();
@@ -518,6 +515,8 @@ private:
 	void eventKeyDownBinding_Stop();
 	void eventKeyDownBinding_Edit();	
 
+	void eventKeyDownBinding_UnmuteAll();
+
 	void eventKeyDownBinding_Open();
 	void eventKeyDownBinding_Save();
 	void eventKeyDownBinding_SaveAs();
@@ -594,6 +593,7 @@ private:
 	friend class SampleLoadChannelSelectionHandler;
 	friend class ZapInstrumentHandler;
 	friend class ToolInvokeHelper;
+	friend class SaveProceedHandler;
 	friend class PanningSettingsContainer;
 	
 	friend class TabManager;
