@@ -1,7 +1,7 @@
 /*
  *  tracker/TrackerInit.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -1654,12 +1654,12 @@ void Tracker::initInstrumentChooser(pp_int32 id, const PPString& buttonText1, co
 
 void Tracker::initAdvEdit()
 {	
-	static pp_int32 buttonIDs[4] = {PP_MESSAGEBOX_BUTTON_USER1, 
+	const pp_int32 buttonIDs[4] =  {PP_MESSAGEBOX_BUTTON_USER1, 
 									PP_MESSAGEBOX_BUTTON_USER2, 
 									PP_MESSAGEBOX_BUTTON_USER3, 
 									PP_MESSAGEBOX_BUTTON_USER4};
 
-	static char* buttonTexts[4] = {"Track",
+	const char* buttonTexts[4] =  {"Track",
 								   "Pattern",
 								   "Song",
 								   "Block"};
@@ -1845,7 +1845,7 @@ void Tracker::initInputContainerDefault(pp_int32 x, pp_int32 y)
 	x3+=button->getSize().width;
 	button = new PPButton(INPUT_BUTTON_BACKLINE, screen, inputControlListener, PPPoint(x3, y2), PPSize(bWidth, bHeight), false, true, true);
 	button->setFont(PPFont::getFont(PPFont::FONT_TINY));
-	button->setText("Bacline");
+	button->setText("Backline");
 	container->addControl(button);	
 
 	// piano test
