@@ -1,7 +1,7 @@
 /*
  *  tracker/TrackerKeyboard.cpp
  *
- *  Copyright 2008 Peter Barth
+ *  Copyright 2009 Peter Barth
  *
  *  This file is part of Milkytracker.
  *
@@ -113,6 +113,7 @@ void Tracker::initKeyBindings()
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_SPACE, KeyModifierALT, &Tracker::eventKeyDownBinding_PlayTrace);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_ESCAPE, 0, &Tracker::eventKeyDownBinding_Stop);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_SPACE, 0, &Tracker::eventKeyDownBinding_Edit);
+	eventKeyDownBindingsMilkyTracker->addBinding('U', KeyModifierSHIFT, &Tracker::eventKeyDownBinding_UnmuteAll);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_F4, KeyModifierALT, &Tracker::eventKeyDownBinding_ExitApplication);
 	
 	eventKeyDownBindingsMilkyTracker->addBinding('Q', KeyModifierCTRL, &Tracker::eventKeyDownBinding_ExitApplication);
@@ -195,6 +196,7 @@ void Tracker::initKeyBindings()
 	// for the die-hard FT2 users
 	eventKeyDownBindingsFastTracker->addBinding(VK_RCONTROL, 0xFFFF, &Tracker::eventKeyDownBinding_PlaySong);
 	eventKeyDownBindingsFastTracker->addBinding(VK_RMENU, 0xFFFF, &Tracker::eventKeyDownBinding_PlayPattern);
+	eventKeyDownBindingsFastTracker->addBinding('U', KeyModifierSHIFT, &Tracker::eventKeyDownBinding_UnmuteAll);
 
 	// Transpose all instruments
 	eventKeyDownBindingsFastTracker->addBinding(VK_F1, KeyModifierSHIFT, &Tracker::eventKeyDownBinding_TransposeAllInsTrackDown);
@@ -470,12 +472,20 @@ void Tracker::eventKeyDownBinding_Edit()
 	}
 }
 
+void Tracker::eventKeyDownBinding_UnmuteAll()
+{
+	if (isActiveEditing())
+		return;
+
+	scopesControl->handleUnmuteAll();
+}
+
 void Tracker::eventKeyDownBinding_Open()
 {
 	if (screen->getModalControl())
 		return;
 
-	loadType(FileTypeSongAllModules);
+	loadType(FileTypes::FileTypeSongAllModules);
 }
 
 void Tracker::eventKeyDownBinding_Save()
@@ -483,52 +493,15 @@ void Tracker::eventKeyDownBinding_Save()
 	if (screen->getModalControl())
 		return;
 
-	if (TrackerConfig::untitledSong.compareTo(moduleEditor->getModuleFileNameFull().stripExtension()) == 0)
-	{
-		eventKeyDownBinding_SaveAs();
-	}
-	else
-	{
-		if (moduleEditor->getSaveType() == ModuleEditor::ModSaveTypeMOD)
-		{
-			pp_uint32 err = moduleEditor->getPTIncompatibilityCode();
-			
-			if (err)
-			{
-				if (savePanel)
-				{
-					delete savePanel;
-					savePanel = NULL;
-				}
-				buildMODSaveErrorWarning((pp_int32)err);
-				return;
-			}
-		}				
-		saveCurrentModuleAsSelectedType();
-	}
+	save();
 }
 
 void Tracker::eventKeyDownBinding_SaveAs()
 {
 	if (screen->getModalControl())
 		return;
-
-	//resetKeyModifier();
-
-	switch (moduleEditor->getSaveType())
-	{
-		case ModuleEditor::ModSaveTypeMOD:
-			saveType(Tracker::FileTypeSongMOD);
-			break;
-
-		case ModuleEditor::ModSaveTypeXM:
-			saveType(Tracker::FileTypeSongXM);
-			break;
-			
-		default:
-			ASSERT(false);
-	}
-
+	
+	saveAs();
 }
 
 void Tracker::eventKeyDownBinding_NextPattern()
