@@ -1,7 +1,7 @@
 /*
  *  tracker/sdl/SDL_Main.cpp
  *
- *  Copyright 2008 Peter Barth, Christopher O'Neill
+ *  Copyright 2009 Peter Barth, Christopher O'Neill
  *
  *  This file is part of Milkytracker.
  *
@@ -64,6 +64,10 @@
 // ---------------------------- Tracker includes ----------------------------
 #include "PPUI.h"
 #include "DisplayDevice_SDL.h"
+#include "DisplayDeviceFB_SDL.h"
+#ifdef __OPENGL__
+#include "DisplayDeviceOGL_SDL.h"  // <-- Experimental, slow
+#endif
 #include "Screen.h"
 #include "Tracker.h"
 #include "PPMutex.h"
@@ -76,13 +80,13 @@
 // --------------------------------------------------------------------------
 
 // SDL surface screen
-SDL_Surface *screen = NULL;
+SDL_Surface*				screen				= NULL;
 SDL_TimerID	timer;
 
 // Tracker globals
 static PPScreen*			myTrackerScreen		= NULL;
 static Tracker*				myTracker			= NULL;
-static PPDisplayDevice*		myDisplayDevice		= NULL;
+static PPDisplayDevice*	    myDisplayDevice		= NULL;
 #ifdef HAVE_LIBASOUND
 static MidiReceiver*		myMidiReceiver		= NULL;
 #endif
@@ -108,7 +112,7 @@ static pp_uint32	rButtonDownStartTime;
 
 static pp_uint32	timerTicker				= 0;
 
-static PPPoint p;
+static PPPoint		p;
 
 // This needs to be visible from outside 
 pp_uint32 PPGetTickCount()
@@ -261,7 +265,16 @@ void gp2xMouseMove()
 }
 #endif
 
-static Uint32 timerCallback(Uint32 interval)
+enum SDLUserEvents
+{
+	SDLUserEventTimer,
+	SDLUserEventLMouseRepeat,
+	SDLUserEventRMouseRepeat,
+	SDLUserEventMidiKeyDown,
+	SDLUserEventMidiKeyUp,
+};
+
+static SDLCALL Uint32 timerCallback(Uint32 interval)
 {
 	timerMutex->lock();
 
@@ -271,11 +284,16 @@ static Uint32 timerCallback(Uint32 interval)
 		return interval;
 	}
 	
+	SDL_UserEvent ev;	
+	ev.type = SDL_USEREVENT;
 
 	if (!(timerTicker % 1))
 	{
-		PPEvent myEvent(eTimer);
-		RaiseEventSerialized(&myEvent);
+		ev.code = SDLUserEventTimer;
+		SDL_PushEvent((SDL_Event*)&ev);		
+		
+		//PPEvent myEvent(eTimer);
+		//RaiseEventSerialized(&myEvent);
 	}
 	
 	timerTicker++;
@@ -287,15 +305,25 @@ static Uint32 timerCallback(Uint32 interval)
 	if (lMouseDown &&
 		(timerTicker - lButtonDownStartTime) > 25)
 	{
-		PPEvent myEvent(eLMouseRepeat, &p, sizeof(PPPoint));		
-		RaiseEventSerialized(&myEvent);
+		ev.code = SDLUserEventLMouseRepeat;
+		ev.data1 = (void*)p.x;
+		ev.data2 = (void*)p.y;
+		SDL_PushEvent((SDL_Event*)&ev);		
+
+		//PPEvent myEvent(eLMouseRepeat, &p, sizeof(PPPoint));		
+		//RaiseEventSerialized(&myEvent);
 	}
 
 	if (rMouseDown &&
 		(timerTicker - rButtonDownStartTime) > 25)
 	{
-		PPEvent myEvent(eRMouseRepeat, &p, sizeof(PPPoint));		
-		RaiseEventSerialized(&myEvent);
+		ev.code = SDLUserEventRMouseRepeat;
+		ev.data1 = (void*)p.x;
+		ev.data2 = (void*)p.y;
+		SDL_PushEvent((SDL_Event*)&ev);		
+
+		//PPEvent myEvent(eRMouseRepeat, &p, sizeof(PPPoint));		
+		//RaiseEventSerialized(&myEvent);
 	}
 
 	timerMutex->unlock();
@@ -304,6 +332,38 @@ static Uint32 timerCallback(Uint32 interval)
 }
 
 #ifdef HAVE_LIBASOUND
+class MidiEventHandler : public MidiReceiver::MidiEventHandler
+{
+public:
+	virtual void keyDown(int note, int volume) 
+	{
+		SDL_UserEvent ev;	
+		ev.type = SDL_USEREVENT;
+		ev.code = SDLUserEventMidiKeyDown;
+		ev.data1 = (void*)note;
+		ev.data2 = (void*)volume;
+		SDL_PushEvent((SDL_Event*)&ev);		
+
+		//globalMutex->lock();
+		//myTracker->sendNoteDown(note, volume);
+		//globalMutex->unlock();
+	}
+
+	virtual void keyUp(int note) 
+	{
+		SDL_UserEvent ev;	
+		ev.type = SDL_USEREVENT;
+		ev.code = SDLUserEventMidiKeyUp;
+		ev.data1 = (void*)note;
+		SDL_PushEvent((SDL_Event*)&ev);		
+
+		//globalMutex->lock();
+		//myTracker->sendNoteUp(note);
+		//globalMutex->unlock();
+	}
+} midiEventHandler;
+
+
 void StopMidiRecording()
 {
 	if (myMidiReceiver)
@@ -319,7 +379,7 @@ void StartMidiRecording(unsigned int devID)
 
 	StopMidiRecording();
 
-	myMidiReceiver = new MidiReceiver(*myTracker, *globalMutex);
+	myMidiReceiver = new MidiReceiver(midiEventHandler);
 
 	if (!myMidiReceiver->startRecording(devID))
 	{
@@ -634,7 +694,6 @@ void translateKeyUpEvent(const SDL_Event& event)
 
 void processSDLEvents(const SDL_Event& event)
 {
-	pp_uint32 mod = SDL_GetModState();
 	pp_uint32 mouseButton = 0;
 
 	switch (event.type)
@@ -678,6 +737,59 @@ void processSDLEvents(const SDL_Event& event)
 	}
 }
 
+void processSDLUserEvents(const SDL_UserEvent& event)
+{
+	switch (event.code)
+	{
+		case SDLUserEventTimer:
+		{
+			PPEvent myEvent(eTimer);
+			RaiseEventSerialized(&myEvent);
+			break;
+		}
+
+		case SDLUserEventLMouseRepeat:
+		{
+			PPPoint p;
+			p.x = (pp_int32)event.data1;
+			p.y = (pp_int32)event.data2;
+			PPEvent myEvent(eLMouseRepeat, &p, sizeof(PPPoint));		
+			RaiseEventSerialized(&myEvent);
+			break;
+		}
+			
+		case SDLUserEventRMouseRepeat:
+		{
+			PPPoint p;
+			p.x = (pp_int32)event.data1;
+			p.y = (pp_int32)event.data2;
+			PPEvent myEvent(eRMouseRepeat, &p, sizeof(PPPoint));		
+			RaiseEventSerialized(&myEvent);
+			break;
+		}
+
+		case SDLUserEventMidiKeyDown:
+		{
+			pp_int32 note = (pp_int32)event.data1;
+			pp_int32 volume = (pp_int32)event.data2;
+			globalMutex->lock();
+			myTracker->sendNoteDown(note, volume);
+			globalMutex->unlock();
+			break;
+		}
+
+		case SDLUserEventMidiKeyUp:
+		{
+			pp_int32 note = (pp_int32)event.data1;
+			globalMutex->lock();
+			myTracker->sendNoteUp(note);
+			globalMutex->unlock();
+			break;
+		}
+
+	}
+}
+
 #ifdef __unix__
 void crashHandler(int signum) 
 {
@@ -715,7 +827,8 @@ void crashHandler(int signum)
 }
 #endif
 
-void initTracker(pp_uint32 bpp, PPDisplayDevice::Orientations orientation, bool swapRedBlue, bool fullScreen, bool noSplash)
+void initTracker(pp_uint32 bpp, PPDisplayDevice::Orientations orientation, 
+				 bool swapRedBlue, bool fullScreen, bool noSplash)
 {
 	/* Initialize SDL */
 	if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0 ) 
@@ -723,7 +836,7 @@ void initTracker(pp_uint32 bpp, PPDisplayDevice::Orientations orientation, bool
 		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
 		exit(1);
 	}
-	atexit(SDL_Quit);
+	// atexit(SDL_Quit);	Not really needed, and needs a wrapper for OS/2
 
 #ifdef __GP2X__
 	if ( SDL_Init(SDL_INIT_JOYSTICK) < 0 || !SDL_JoystickOpen(0)) 
@@ -771,14 +884,22 @@ void initTracker(pp_uint32 bpp, PPDisplayDevice::Orientations orientation, bool
 	myTracker = new Tracker();
 
 	PPSize windowSize = myTracker->getWindowSizeFromDatabase();
- 	if(!fullScreen) fullScreen = myTracker->getFullScreenFlagFromDatabase();
+ 	if (!fullScreen) 
+		fullScreen = myTracker->getFullScreenFlagFromDatabase();
+	pp_int32 scaleFactor = myTracker->getScreenScaleFactorFromDatabase();
+
 #ifdef __LOWRES__
 	windowSize.width = DISPLAYDEVICE_WIDTH;
 	windowSize.height = DISPLAYDEVICE_HEIGHT;
 #endif
 
-	myDisplayDevice = new PPDisplayDevice(screen, windowSize.width, windowSize.height, bpp, fullScreen, orientation, swapRedBlue);
-
+#ifdef __OPENGL__
+	myDisplayDevice = new PPDisplayDeviceOGL(screen, windowSize.width, windowSize.height, 1, bpp, fullScreen, orientation, swapRedBlue);
+#else
+	myDisplayDevice = new PPDisplayDeviceFB(screen, windowSize.width, windowSize.height, scaleFactor, 
+											bpp, fullScreen, orientation, swapRedBlue);
+#endif 
+	
 	myDisplayDevice->init();
 
 	myTrackerScreen = new PPScreen(myDisplayDevice, myTracker);
@@ -855,8 +976,10 @@ int main(int argc, char *argv[])
 	PPDisplayDevice::Orientations orientation = PPDisplayDevice::ORIENTATION_NORMAL;
 	bool swapRedBlue = false, fullScreen = false, noSplash = false;
 	bool recVelocity = false;
+	
 	// Parse command line
-	while ( argc > 1 ) {
+	while ( argc > 1 ) 
+	{
 		--argc;
 		if ( strcmp(argv[argc-1], "-bpp") == 0 ) 
 		{
@@ -901,13 +1024,17 @@ int main(int argc, char *argv[])
 		{
 			recVelocity = true;
 		}
-		else {
+		else 
+		{
 unrecognizedCommandLineSwitch:
-			if(argv[argc][0] == '-') {
+			if (argv[argc][0] == '-') 
+			{
 				fprintf(stderr, 
-						"Usage: %s [-bpp N] [-swap] [-orientation NORMAL|ROTATE90CCW|ROTATE90CW] [-fullscreen] [-nosplash] [-nonstdbk] [-recvelocity]\n", argv[0]);
+						"Usage: %s [-bpp N] [-swap] [-orientation NORMAL|ROTATE90CCW|ROTATE90CW] [-fullscreen] [-nosplash] [-nonstdkb] [-recvelocity]\n", argv[0]);
 				exit(1);
-			} else {
+			} 
+			else 
+			{
 				loadFile = argv[argc];
 			}
 		}
@@ -952,7 +1079,8 @@ unrecognizedCommandLineSwitch:
 	done = 0;
 	while (!done && SDL_WaitEvent(&event)) 
 	{
-		switch (event.type) {
+		switch (event.type) 
+		{
 			case SDL_QUIT:
 				exitSDLEventLoop(false);
 				break;
@@ -973,6 +1101,10 @@ unrecognizedCommandLineSwitch:
 				break;
 			}
 
+			case SDL_USEREVENT:
+				processSDLUserEvents((const SDL_UserEvent&)event);
+				break;
+
 			default:
 				processSDLEvents(event);
 				break;
@@ -998,6 +1130,7 @@ unrecognizedCommandLineSwitch:
 	myTracker = NULL;
 	delete myTrackerScreen;
 	myTrackerScreen = NULL;
+	delete myDisplayDevice;
 	globalMutex->unlock();
 	timerMutex->unlock();
 	SDL_Quit();
